import Skypager from '../index'
import Script from './script'
import partial from 'lodash/partial'

/**
 * HelperScript assets are the scripts which get used
 * as skypager helpers.
 *
 * - actions
 * - importers
 * - exporters
 * - plugins
 * - models
 * - renderers
 *
*/

const GroupNames = ['actions','exporters','importers','models','plugins','renderers']

export class HelperScript extends Script {

  /**
   * Creates shortcuts on the helpers collection
   * for accessing the different types of helpers by group name.
   * e.g. actions, models, exporters
  */
  static decorateCollection (collection) {
    const scope = partial(buildScope, collection)

    GroupNames.forEach(groupName => {
      scope(groupName, () => {
        return collection.where(
          s => s.groupName === groupName && !s.isLoader
        )
      })
    })

    scope('registryLoaders', () =>
      collection.where(script => {
        return script.depth === 1 && script.uri.match(/index.js$/)
      })
    )

    Object.defineProperty(collection, 'compiledPath', {
      enumerable: false,
      configurable: true,
      get: function () {
        return collection.paths.absolute.replace(/\/src/, '/lib')
      }
    })
  };

  /**
   * Loads this helper into the registry.
   *
   * This method will ensure that the DSL methods a particular type
   * of helper may rely on to generate the interface definition are
   * in scope for the script.
   */
  register(host = this.registry) {
    if (host && !this.isLoader) {
      host.load(
        require(this.paths.absolute), {
          id: this.helperId,
          uri: this.paths.absolute
        }
      )
    } else if (host && this.isLoader) {
      host.runLoader(
        require(this.paths.absolute)
      )
    }
  }

  get uri() {
    let absolute = this.paths.absolute

    return absolute.match(/^\//)
      ? absolute
      : `${ this.collection.root }/${ absolute }`
  }

  get helperId() {
    return this.id.replace(`${ this.groupName }/`, '')
  }

  get isLoader() {
     return this.depth === 1 && this.isIndex
  }

  get registry () {
    return this.project
      ? this.project.get('registries.' + this.groupName)
      : Skypager.registries[this.groupName]
  }
}

export default HelperScript


function buildScope (collection, name, get) {
  Object.defineProperty(collection, name, {
    enumerable: false,
    configurable: false,
    get
  })
}
